// *****************************************************************
// ****                      Synth CORE B                       ****
// ****                  www.tolaemon.com/scoreb                ****
// ****                       Source code                       ****
// ****                       13/11/2011                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/scoreb             ****
// ****                                                         ****
// **** Parts of this code have been taken from the demo source ****
// **** code of EVK1100 Atmel's evaluation board				****
// *****************************************************************

#include <avr32/io.h>
#include "board.h"
#include "gpio.h"

// scheduler includes.
// #include "FreeRTOS.h"
// #include "queue.h"
// #include "task.h"

// synth core B includes
// #include "defines.h"
#include "lcd_hd44780.h"

static char LCD_HD44780_buffer[LCD_HD44780_NUM_COLS];


// Synth Core  10-10-2012  Tolaemon
// Function: void LCD_HD44780_active_pause()
// Implements an active pause with a bucle. It is no efficient but the OS TaskDelay only gives
// 1ms ressolution and it makes refreshing the LCD very slow, so I had to use this kind of pause
// Receives:
// Returns:
void LCD_HD44780_active_pause(){
	unsigned int ui_aux=0;

	for (ui_aux=0;ui_aux<100;ui_aux++){
		asm ("nop");
		asm ("nop");
	}//for

}//LCD_HD44780_active_pause


// Synth Core  10-12-2011  Tolaemon
// Function: void LCD_HD44780_send_data_byte()
// Executes the sequence that makes the LCD read the bits set into the data bus as Data
// Byte. The main difference with the send_command_byte is in the RS bit. When the RS bit
// is 1 bits set into the data bus correspond to a Data Byte, on the other side when is 0
// it corresponds to a Command Byte
// Receives:
// Returns:
void LCD_HD44780_send_data_byte(){

	gpio_local_set_gpio_pin(LCD_HD44780_IO_RS);// RS bit is set to 1
	gpio_local_clr_gpio_pin(LCD_HD44780_IO_E); // E bit is set to 0
	LCD_HD44780_active_pause(); // pause
	gpio_local_set_gpio_pin(LCD_HD44780_IO_E); // E bit is also set to 1
	LCD_HD44780_active_pause(); // pause
	gpio_local_clr_gpio_pin(LCD_HD44780_IO_E); // E bit is set to 0

}// LCD_HD44780_send_data_byte



// Synth Core  10-12-2011  Tolaemon
// Function: void LCD_HD44780_send_command_byte()
// Executes the sequence that makes the LCD read the bits set into the data bus as Command
// Byte. The main difference with the send_command_byte procedure is in the RS bit. When
// the RS bit is 1 bits set into the data bus correspond to a Data Byte, on the other side
// when is 0 it corresponds to a Command Byte
// Receives:
// Returns:
void LCD_HD44780_send_command_byte(){

	gpio_local_clr_gpio_pin(LCD_HD44780_IO_RS);// RS bit is set to 0
	gpio_local_clr_gpio_pin(LCD_HD44780_IO_E); // E bit is also set to 0
	LCD_HD44780_active_pause(); // pause
	gpio_local_set_gpio_pin(LCD_HD44780_IO_E); // E bit is also set to 1
	LCD_HD44780_active_pause(); // pause
	gpio_local_clr_gpio_pin(LCD_HD44780_IO_E); // E bit is set to 0

}// LCD_HD44780_send_command_byte



// Synth Core  10-12-2011  Tolaemon
// Function: void LCD_HD44780_set_data_byte(portCHAR data_byte)
// checks each of the bits of the byte we want to write to the LCD Data lines
// and modifies the state of the corresponding IO lines
// Receives:
//   portCHAR data_byte: with the byte we want to write into the LCD Data IO lines
// Returns:
void LCD_HD44780_set_data_byte(portCHAR data_byte){

	if ((data_byte&0x01)==0){
		gpio_local_clr_gpio_pin(LCD_HD44780_IO_D0);
	}else{
		gpio_local_set_gpio_pin(LCD_HD44780_IO_D0);
	}//if
	if ((data_byte&0x02)==0){
		gpio_local_clr_gpio_pin(LCD_HD44780_IO_D1);
	}else{
		gpio_local_set_gpio_pin(LCD_HD44780_IO_D1);
	}//if
	if ((data_byte&0x04)==0){
		gpio_local_clr_gpio_pin(LCD_HD44780_IO_D2);
	}else{
		gpio_local_set_gpio_pin(LCD_HD44780_IO_D2);
	}//if
	if ((data_byte&0x08)==0){
		gpio_local_clr_gpio_pin(LCD_HD44780_IO_D3);
	}else{
		gpio_local_set_gpio_pin(LCD_HD44780_IO_D3);
	}//if
	if ((data_byte&0x10)==0){
		gpio_local_clr_gpio_pin(LCD_HD44780_IO_D4);
	}else{
		gpio_local_set_gpio_pin(LCD_HD44780_IO_D4);
	}//if
	if ((data_byte&0x20)==0){
		gpio_local_clr_gpio_pin(LCD_HD44780_IO_D5);
	}else{
		gpio_local_set_gpio_pin(LCD_HD44780_IO_D5);
	}//if
	if ((data_byte&0x40)==0){
		gpio_local_clr_gpio_pin(LCD_HD44780_IO_D6);
	}else{
		gpio_local_set_gpio_pin(LCD_HD44780_IO_D6);
	}//if
	if ((data_byte&0x80)==0){
		gpio_local_clr_gpio_pin(LCD_HD44780_IO_D7);
	}else{
		gpio_local_set_gpio_pin(LCD_HD44780_IO_D7);
	}//if

}//LCD_HD44780_set_data_byte



// Synth Core  10-12-2011  Tolaemon
// Function: void LCD_HD44780_init()
// Switchs on and configures the LCD
// Receives:
// Returns:
void LCD_HD44780_init(){

	// TODO maybe the port initialization ( pins and direction ) should be done here

	LCD_HD44780_set_data_byte(0x0C);// switch on the LCD command and disable cursor
	LCD_HD44780_send_command_byte();
	LCD_HD44780_set_data_byte(0x38);// configue the LCD command
	LCD_HD44780_send_command_byte();

}//LCD_HD44780_init



// Synth Core  10-12-2011  Tolaemon
// Function:void LCD_HD44780_clear()
// Receives:
// Returns:
void LCD_HD44780_clear(){

	LCD_HD44780_set_data_byte(0x01);
	LCD_HD44780_send_command_byte();

}// LCD_HD44780_clear



// Synth Core  10-12-2011  Tolaemon
// Function: void LCD_HD44780_write(portCHAR * string_to_write, portCHAR line_numb)
// Receives:
//  portCHAR * string_to_write: with the string we want to write into the LCD
//  portCHAR line_numb: with the LCD line number where we want to write the string (1,2,3,4)
// Returns:
void LCD_HD44780_write(portCHAR * string_to_write, portCHAR line_numb){

	LCD_HD44780_clear_buffer();
	LCD_HD44780_set_string_buffer(string_to_write);
	LCD_HD44780_write_buffer_to_line(line_numb);

}// LCD_HD44780_write



// Synth Core  10-12-2011  Tolaemon
// Function: portCHAR LCD_HD44780_get_char_buffer(portSHORT position)
// Receives:
//  portSHORT position:
// Returns:
portCHAR LCD_HD44780_get_char_buffer(portSHORT position){
	portSHORT i;

	i=0;
	if ( (position>0) && (position<LCD_HD44780_NUM_COLS) ){
		i= LCD_HD44780_buffer[position];
	}//if
	return (char)i;

}// LCD_HD44780_get_char_buffer



// Synth Core  10-12-2011  Tolaemon
// Function: void LCD_HD44780_set_char_buffer(char character,portSHORT position)
// Receives:
//  char character : with the character we want to set
//  portSHORT position : the buffer position where we want to set the character in
// Returns:
void LCD_HD44780_set_char_buffer(char character,portSHORT position){

	if (position<LCD_HD44780_NUM_COLS) {
		LCD_HD44780_buffer[position]=character;
	}//if

}// LCD_HD44780_set_char_buffer



// Synth Core  10-12-2011  Tolaemon
// Function: void LCD_HD44780_clear_buffer()
// Clears the content of the buffer
// Receives:
// Returns:
void LCD_HD44780_clear_buffer(){
	portSHORT i;

	for (i=0;i<LCD_HD44780_NUM_COLS;i++){
		LCD_HD44780_buffer[i]=' ';
	}//for

}// LCD_HD44780_reset_buffer



// Synth Core  10-12-2011  Tolaemon
// Function: void LCD_HD44780_write_buffer_to_line(char line_numb)
// Writes the buffer content into the LCD specified line_numb
// Receives:
//  line_numb: the LCD line where we want to write the buffer content in
// Returns:
void LCD_HD44780_write_buffer_to_line(char line_numb){
	portSHORT i;

	// sets LCDs cursor into the specified line_num
	switch (line_numb){
		case 1:
			LCD_HD44780_set_data_byte(0x80); // sets the address 00 which corresponds to line 1
			break;
		case 2:
			LCD_HD44780_set_data_byte(0xA9);// sets the address 41 which corresponds to line 2
			break;
		default:
			LCD_HD44780_set_data_byte(0x80);// by default it sets the address 00 which corresponds to line 1
			break;
	}// Switch
	LCD_HD44780_send_command_byte();

	for (i=0;i<LCD_HD44780_NUM_COLS;i++){
		LCD_HD44780_set_data_byte(LCD_HD44780_buffer[i]);// each char stored into the buffer is set into the data port
		LCD_HD44780_send_data_byte();
	};// for

}// LCD_HD44780_write_buffer_to_line



// Synth Core  10-12-2011  Tolaemon
// Function: void LCD_HD44780_set_string_buffer(portCHAR * string_to_set)
// Receives a string and sotres it into the line LCD buffer
// Receives:
//  portCHAR * string_to_set: with the string we want tos et into the buffer
// Returns:
void LCD_HD44780_set_string_buffer(portCHAR * string_to_set){
	portSHORT i;

	i=0;
	while ( (i<LCD_HD44780_NUM_COLS) && (string_to_set[i]!='\0') ){
		LCD_HD44780_buffer[i]=string_to_set[i];
		i++;
	}//for

}// LCD_HD44780_set_string_buffer

